program distance

  ! Written by M. Cococcioni

  implicit none

  real*8 pos(3,2000),    & ! coordinates of the atoms
         dist(2000),     &
         distord(2000),  &
         alat(3,3),      & ! lattice vectors (in celldm units)
         posi(3)
  real*8 distm,          &
         pos0(3),        &
         pos_sc(3,2700), &
         celldm            ! lattice parameter (in Bohr)

  integer nat, &         ! number of atoms
          nat_sc, &
          i0, &          ! i0 identifies the atom, from which the distances are computed 
          i, j, l, k, &
          i1, i2, i3, iord(2000), lmin, &
          n1, n2, n3

  character*4 spec(2000),    & ! atomic positions in crystal coordinates
              specord(2000), &
              spec0            ! name of the reference atom

  logical written_once

  write(6,*) 'please enter nat, i0 and spec0'
  read(5,*) nat, i0, spec0 
  !
  ! Read the data about the system
  !
  open(11,file='unitcell_and_atomic_positions.in',status='unknown')
  !
  read(11,*) celldm
  !
  do i = 1,3
     read(11,*) (alat(j,i),j=1,3)
     alat(:,i) = alat(:,i) * celldm
  enddo
  !
  do i = 1, nat
     read(11,*) spec(i), (pos(j,i),j=1,3)
  enddo
  !
  close(11)
  !
  ! Open the file for writing of the reshuffled atomic coordinates
  !
  open(12,file='unitcell_and_atomic_positions.out',status='unknown')
  !
  ! Compute the distances
  !     
  pos0(:) = 0.d0
  !
  do j = 1, 3
     pos0(j) = alat(j,1) * pos(1,i0) + &
               alat(j,2) * pos(2,i0) + &
               alat(j,3) * pos(3,i0)
  enddo
  !
  do i = 1, nat
     !
     distm = 1000.d0
     !
     do i1 = -1, 1
        do i2 = -1, 1
           do i3 = -1, 1
              !
              posi(:) = 0.d0
              !
              do j = 1, 3
                 posi(j) = posi(j) + alat(j,1) * (pos(1,i) + dfloat(i1)) + &
                                     alat(j,2) * (pos(2,i) + dfloat(i2)) + &
                                     alat(j,3) * (pos(3,i) + dfloat(i3)) 
              enddo
              !
              dist(i) = 0.d0
              !
              do j = 1, 3
                 dist(i) = dist(i) + (posi(j)-pos0(j))**2
              enddo
              !
              dist(i) = sqrt(dist(i))
              dist(i) = min(dist(i),distm)
              !
              if (dist(i).lt.distm) distm = dist(i)
              !
           enddo ! i3
        enddo ! i2
     enddo ! i1
     !
     if (i.ne.i0) then
        !write(12,'(a4,3(x,f14.8))') spec(i), (pos(j,i)-pos(j,i0), j=1,3)
        write(12,'(a4,3(x,f16.10))') spec(i), (pos(j,i), j=1,3)
     else
        !write(12,'(a4,3(x,f14.8))') spec0,(pos(j,i)-pos(j,i0),j=1,3)
        write(12,'(a4,3(x,f16.10))') spec0,(pos(j,i),j=1,3)
     endif
     !
  enddo
  !
  ! do i = 1,nat
  !    write(12,*) i, spec(i), dist(i)
  ! enddo
  ! write(12,*)
  !
  ! Order the distances
  !
  l = 1
  !
  do i = 1, nat
     if (dist(i).gt.0.d0) then
        distord(1) = dist(i)
        specord(1) = spec(i)
        iord(1) = i
        go to 3
      end if
  end do
  !
3 continue
  !
  do i = 1, nat
     if (dist(i).gt.distord(1)) then
        l = l + 1
        distord(l) = dist(i)
        specord(l) = spec(i)
        iord(l) = i
        go to 5
     end if
  end do
  !
  ! distord(1) e' la dist maggiore
  !    
  l = l + 1
  !
  distord(l) = distord(1)
  specord(l) = specord(1)
  iord(l) = iord(1)
  !
  do i = 1,nat
     if (dist(i).gt.0.d0.and.dist(i).lt.distord(l)) then
        distord(1) = dist(i)
        specord(1) = spec(i)
        iord(1) = i
        go to 5
      end if
  end do
  !
5    continue
  !
! write(6,*) iord(1), iord(l), specord(1), specord(l)
  ! 
  do i = 1, nat
     !
     if (dist(i).gt.distord(l)) then
        l = l+1
        distord(l) = dist(i)
        specord(l) = spec(i)
        iord(l) = i
      elseif (dist(i).le.distord(1).and.i.ne.iord(1)) then
        lmin = 2
        l = l+1
        !
        do k = l,lmin,-1
           distord(k) = distord(k-1)
           specord(k) = specord(k-1)
           iord(k) = iord(k-1)
        enddo
        !
        distord(1) = dist(i)
        specord(1) = spec(i)
        iord(1) = i
      else 
        do k = l-1,1,-1
         if (dist(i).gt.distord(k).and.dist(i).le.distord(k+1)) then
           if (i.ne.iord(k+1)) then
!          if (dist(i).gt.distord(k).and.dist(i).le.distord(k+1)) then
              lmin = k + 2
              l = l+1
              do j = l,lmin,-1
                 distord(j) = distord(j-1)
                 specord(j) = specord(j-1)
                 iord(j) = iord(j-1)
              end do
              distord(k+1) = dist(i)
              specord(k+1) = spec(i)
              iord(k+1) = i !iord(i)
              go to 7
           end if
         end if
        end do
        !
7       continue
        !
      end if
      !
  end do
  !
  do i = 1, l
     write(12,*) iord(i), specord(i), distord(i)
  end do
  !
  ! For the construction of a supercell
  !
  written_once = .false.
  !
  write(6,*) 'n1 n2 n3 ?'
  read(5,*) n1, n2, n3
  !
  nat_sc = 0
  !
  do i1 = 1, n1
     do i2 = 1, n2
        do i3 = 1, n3
           !
           do i = 1, nat
              !
              nat_sc = nat_sc + 1
              !
              !pos_sc(1,nat_sc) = (pos(1,i)-pos(1,i0)+dfloat(i1-1)) / dfloat(n1)
              !pos_sc(2,nat_sc) = (pos(2,i)-pos(2,i0)+dfloat(i2-1)) / dfloat(n2)
              !pos_sc(3,nat_sc) = (pos(3,i)-pos(3,i0)+dfloat(i3-1)) / dfloat(n3)
              !
              pos_sc(1,nat_sc) = (pos(1,i)+dfloat(i1-1)) / dfloat(n1)
              pos_sc(2,nat_sc) = (pos(2,i)+dfloat(i2-1)) / dfloat(n2)
              pos_sc(3,nat_sc) = (pos(3,i)+dfloat(i3-1)) / dfloat(n3)
              !
              IF (.not.written_once) THEN
                 !
                 if (i.ne.i0) then
                    write(12,'(a4,3(x,f16.10))') spec(i), (pos_sc(j,nat_sc),j=1,3)
                 else
                    write(12,'(a4,3(x,f16.10))') spec0, (pos_sc(j,nat_sc),j=1,3)
                    written_once=.true.
                 endif
                 !
              ELSE
                 write(12,'(a4,3(x,f16.10))') spec(i), (pos_sc(j,nat_sc),j=1,3)
              ENDIF
              !
           enddo
           !
        enddo
     enddo
  enddo
  !
  close(12)
  !
  stop
  !    
end program distance 
