!---------------------------------------------------------------------------
program resp_mat
!---------------------------------------------------------------------------
      !
      ! This program constructs the response matrices to compute the Hubbard 
      ! parameetrs U and V.
      ! The responses dnI/dalphaJ are written in the files dn_(1:nat)_da_(1:ntyp) 
      ! for the scf response and dn0_(1:nat)_da_(1:ntyp) for the bare one.
      ! This program must be run in serial; it uses LAPACK routines.
      !
      ! Written by M. Cococcioni; modified by I. Timrov (2020).
      !
      implicit none
      !
      real(kind=8)   primv(3,3), atp_cry(3,400), atp_car(3,400),        &
                     d_cry(3), d_car(3), sum, sum0,                     &
                     daux, daux0, rm, alat, atp_cry_refc(3,400)
      real(kind=8), allocatable ::  chi(:,:), chi0(:,:), chim1(:,:),    &
      &                             chi0m1(:,:), alpha(:),              &
      &                             nn(:), dist(:,:), dn(:), da(:),     &
      &                             chi_aux(:,:), chi0_aux(:,:)
      !
      integer  ntyp, na(50), nalfa, nat, nat0, nat_ref
      integer  i, j, k, l, m, n, l1, l2, l3, i0, ityp
      integer, allocatable :: typ(:), spin(:), rmi(:), spin_ref(:), typ_ref(:)
      integer donetyp(50)
      !
      character*30 filepos, back, filednda
      character*30, allocatable :: filename(:,:), filename0(:,:)
      character*4 symbol
      logical :: magn, condition
      !
      namelist /input_mat/ ntyp, na, nalfa, filepos, back,  &
                           filednda, magn, typ, nat_ref
      !
      read(5,input_mat)
      !
      nat = 0
      do i = 1,ntyp
         nat = nat + na(i)
      end do
      !
      allocate(chi(nat+1,nat+1))
      allocate(chi0(nat+1,nat+1))
      allocate(chi_aux(nat,nat))
      allocate(chi0_aux(nat,nat))
      allocate(chim1(nat,nat))
      allocate(chi0m1(nat,nat))
      allocate(alpha(nalfa))
      allocate(nn(nalfa))
      allocate(dist(nat,nat))
      allocate(typ(nat))
      allocate(typ_ref(nat_ref))
      allocate(filename(nat,nat))
      allocate(filename0(nat,nat))
      allocate(spin(nat))
      allocate(spin_ref(nat_ref))
      !
      spin(:) = 0
      !
      open(11,file=filepos,status='unknown')
      !
      read(11,*) alat
      ! 
      ! Read the primitive lattice vectors
      !
      do i = 1, 3
         !
         read(11,*) (primv(j,i),j=1,3)
         !
         do j = 1,3
            primv(j,i) = primv(j,i) * alat
         enddo
         !
      enddo
      !
      ! Read the atomic coordinates, atomic type (and spin)
      ! 
      if (magn) then
         do i = 1, nat_ref
            read(11,*) symbol, (atp_cry_refc(k,i),k=1,3), typ_ref(i), spin_ref(i)
         enddo
      else
         do i = 1, nat_ref
            read(11,*) symbol, (atp_cry_refc(k,i),k=1,3), typ_ref(i)
         enddo
      endif
      !
      if (magn) then
        !
        nat = 0
        do i = 1, nat_ref
           nat = nat + 1
           do k = 1,3
              atp_cry(k,nat) = atp_cry_refc(k,i)
           enddo
           spin(nat) = spin_ref(i)
           typ(nat)  = typ_ref(i)
        enddo
        !
      else
        !
        nat = 0
        do i = 1, nat_ref
           nat = nat + 1
           do k = 1,3
              atp_cry(k,nat) = atp_cry_refc(k,i)
           enddo
           typ(nat) = typ_ref(i)
        enddo
        !
      endif
      !
      close(11)
      !
      !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      !
      open(11,file=filednda,status='unknown')
      !
      donetyp(:) = 0
      !
      do i = 1, nat
         !
         if (donetyp(typ(i)).eq.0) then
            !
            do j = 1, nat
               ! 
               read(11,*) filename(i,j), filename0(i,j)
               ! 
            enddo
            !
            donetyp(typ(i)) = 1
            ! 
         endif
         !
      enddo
      !
      close(11)
      !
      chi(:,:)  = 0.d0
      chi0(:,:) = 0.d0
      !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!! Construct the columns (one per atomic type) of the response matrices !!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      !
      nat0 = 0
      donetyp(:) = 0
      !
      DO j = 1, nat
         !
         IF (donetyp(typ(j)).eq.0) THEN
            !
            DO i = 1, nat
               !
               ! Calculate chi
               !
               open(11,file=filename(j,i),status='unknown')
               do k = 1, nalfa
                  read(11,*) alpha(k), nn(k)
               enddo
               chi(i,j) = (nn(nalfa)-nn(1))/(alpha(nalfa)-alpha(1))
               close(11)
               !
               ! Calculate chi0
               ! 
               open(11,file=filename0(j,i),status='unknown')
               do k = 1, nalfa
                  read(11,*) alpha(k), nn(k)
               end do
               chi0(i,j) = (nn(nalfa)-nn(1))/(alpha(nalfa)-alpha(1))
               close(11)
               !
            ENDDO
            !
            donetyp(typ(j)) = 1
            !
         ENDIF
         !
      ENDDO
      !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!! Trasform atomic coordinates from crystal to cartesian framework !!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      !
      atp_car(:,:) = 0.d0
      do i = 1, nat
         do j = 1, 3
            do k = 1, 3
               atp_car(j,i) = atp_car(j,i) + primv(j,k) * atp_cry(k,i)
            enddo
         enddo
      enddo
      !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!! Compute distances between atoms !!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      !
      do i = 1, nat
         !
         dist(i,i) = 0.d0
         !
         do j = i+1, nat
            !
            dist(i,j) = 1000.d0
            !
            do l1 = -1, 1
               do l2 = -1, 1
                  do l3 = -1, 1
                     !
                     daux = 0.d0
                     !
                     do k = 1, 3
                        !
                        daux = daux + ( atp_car(k,i) - atp_car(k,j)  &
                                       - dfloat(l1) * primv(k,1)     &
                                       - dfloat(l2) * primv(k,2)     &
                                       - dfloat(l3) * primv(k,3) )**2
                        !
                     enddo
                     !
                     daux = dsqrt(daux)
                     dist(i,j) = min(dist(i,j),daux)
                     !
                  enddo
               enddo
            enddo
            !
            dist(j,i) = dist(i,j)
            !
         enddo
      enddo
      !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!! Check if there are similar elements in chi (chi0) matrix !!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      !
      allocate (rmi(nat))
      DO j = 1, nat
         !
         DO l = 1, nat
            !
            ! For chi
            !
            IF (chi(j,l).ne.0.d0) THEN
               !
               nat0 = 1
               rm = chi(j,l)
               rmi(nat0) = j
               !
               ! For a given column of chi, check if there are
               ! similar by value elements.
               !
               do k = 1, nat 
                  !
                  if (typ(k).eq.typ(j) .and. &
                      abs(dist(k,l)-dist(j,l)).le.6.d-4 .and. &
                      dist(k,l).gt.0.d0 .and. &
                      dist(j,l).gt.0.d0 .and. &
                      k.ne.j .and. &
                      chi(k,l).ne.0.d0 .and. &
                      spin(k).eq.spin(j)) then
                      !
                      nat0 = nat0 + 1
                      rmi(nat0) = k
                      rm = rm + chi(k,l)
                      !
                  endif
                  !
               enddo
               !
               IF ( nat0 > 1 ) THEN
                  do k = 1, nat0
                     chi(rmi(k),l) = rm/DBLE(nat0)
                  enddo
               ENDIF
               !
            ENDIF
            !
            ! For chi_0
            !
            IF (chi0(j,l).ne.0.d0) THEN
               !
               nat0 = 1
               rm = chi0(j,l)
               rmi(nat0) = j
               !
               ! For a given column of chi, check if there are
               ! similar by value elements.
               !
               do k = 1, nat
                  !
                  condition = typ(k).eq.typ(j) .and. &
                              abs(dist(k,l)-dist(j,l)).le.6.d-4 .and. &
                              dist(k,l).gt.0.d0 .and. &
                              dist(j,l).gt.0.d0 .and. &
                              k.ne.j .and. &
                              chi0(k,l).ne.0.d0 .and. &
                              spin(k).eq.spin(j)
                  ! 
                  if (condition) then
                      !
                      nat0 = nat0 + 1
                      rmi(nat0) = k
                      rm = rm + chi0(k,l)
                      !
                  endif
                  !
               enddo
               !
               IF ( nat0 > 1 ) THEN
                  do k = 1, nat0
                     chi0(rmi(k),l) = rm/DBLE(nat0)
                  enddo
               ENDIF
               !
            ENDIF
            !
         ENDDO
         !
      ENDDO
      deallocate(rmi)
      !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!! Use the symmetry to reconstruct the missing elements of the chi (chi0) matrix !!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      !
      DO j = 1, nat
        DO l = 1, nat
        !
        IF ( chi(j,l).EQ.0.d0 ) THEN
           !
           DO k = 1, nat
              IF ( typ(k).EQ.typ(l) ) THEN
                 !
                 DO i = 1, nat
                    IF ( typ(i).EQ.typ(j) ) THEN
                     !
                     IF (magn) THEN
                       IF (chi(i,k).NE.0.0d0 .AND. &
                           ABS(dist(i,k)-dist(j,l)).LE.6.d-4  .AND. &
                           spin(j)*spin(l).EQ.spin(i)*spin(k)) THEN
                           chi(j,l)  = chi(i,k)
                           chi0(j,l) = chi0(i,k)
                           GO TO 35
                       ENDIF
                       IF (chi(k,i).NE.0.0d0 .AND. &
                           ABS(dist(k,i)-dist(j,l)).LE.6.d-4  .AND. &
                           spin(j)*spin(l).EQ.spin(k)*spin(i)) THEN
                           chi(j,l)  = chi(k,i)
                           chi0(j,l) = chi0(k,i)
                           GO TO 35
                       ENDIF
                     ELSE
                       !
                       IF (chi(i,k).NE.0.0d0 .AND. &
                           ABS(dist(i,k)-dist(j,l)).LE.6.d-4) THEN
                           chi(j,l)  = chi(i,k)
                           chi0(j,l) = chi0(i,k)
                           GO TO 35
                       ENDIF
                       IF (chi(k,i).NE.0.0d0 .AND. &
                           ABS(dist(k,i)-dist(j,l)).LE.6.d-4) THEN
                           chi(j,l)  = chi(k,i)
                           chi0(j,l) = chi0(k,i)
                           GO TO 35
                       ENDIF 
                       ! 
                     ENDIF
                     !
                    ENDIF
                 ENDDO
                 !
              ENDIF
           ENDDO
           !
        ENDIF
        !
35 CONTINUE
        !
        ENDDO
      ENDDO
      !
      ! Check that all elements were found
      !
      IF (ANY(chi(1:nat,1:nat).EQ.0.0d0) .OR. ANY(chi0(1:nat,1:nat).EQ.0.0d0)) THEN
         !
         WRITE(6, '(/5x,"Existing distances between couples of atoms:"/)')
         DO i = 1, nat
            DO j = 1, nat
               WRITE(6, '(5x,"i=",2x,i4,2x,"j=",2x,i4,2x,"dist= ",f10.6)') &
               i, j, dist(i,j)
            ENDDO
         ENDDO
         !
         DO i = 1, nat
            DO j = 1, nat
               IF (chi(i,j).EQ.0.0d0) &
                  WRITE(6,*) "Missing chi element for: (i,j)=", i, j, "dist=", dist(i,j) 
               IF (chi0(i,j).EQ.0.0d0) &
                  WRITE(6,*) "Missing chi0 element for: (i,j)=", i, j, "dist=", dist(i,j)
            ENDDO
         ENDDO
         STOP
      ENDIF
      !
      ! Symmetrization
      !
      do i = 1, nat
         do j = 1, nat
            !
            chi(i,j) = 0.5d0*(chi(i,j)+chi(j,i))
            chi(j,i) = chi(i,j)
            !
            chi0(i,j) = 0.5d0*(chi0(i,j)+chi0(j,i))
            chi0(j,i) = chi0(i,j)
            !
         enddo
      enddo
 
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 
!!!!!!!!!!!!!!!!! Background (optional) !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
     
      IF (back.eq.'neutral') THEN
         !
         sum = 0.d0
         sum0 = 0.d0
         !
         do i = 1, nat
            !
            chi(i,nat+1) = 0.d0
            chi0(i,nat+1) = 0.d0
            !
            do j = 1, nat
               !
               chi(i,nat+1)  = chi(i,nat+1)  - chi(i,j)
               chi0(i,nat+1) = chi0(i,nat+1) - chi0(i,j)
               !
            enddo
            !
            chi(nat+1,i)  = chi(i,nat+1)
            chi0(nat+1,i) = chi0(i,nat+1)
            !
            sum  = sum  - chi(i,nat+1)
            sum0 = sum0 - chi0(i,nat+1)
            !
         enddo
         !
         chi(nat+1,nat+1)  = sum
         chi0(nat+1,nat+1) = sum0
         !
         do i = 1, nat+1
            do j = 1, nat+1
               !
               chi(i,j)  = chi(i,j)  + 0.01
               chi0(i,j) = chi0(i,j) + 0.01
               !
            end do
         enddo
         !
      ELSEIF (back.eq.'no'.or.back.eq.' ') THEN
         !
         chi(nat+1,nat+1)  = 1.d0
         chi0(nat+1,nat+1) = 1.d0
         !
      ENDIF
      !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!! Invert the matrices chi and chi0 !!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      !
      IF (back.eq.'neutral' .or. back.eq.'computed') THEN
         ! 
         CALL invmat_loc(chi,chim1,nat+1)
         CALL invmat_loc(chi0,chi0m1,nat+1)
         !
         if (back.eq.'neutral') then
            do i = 1, nat+1
               do j = 1, nat+1
                  chi(i,j)  = chi(i,j)  - 0.01
                  chi0(i,j) = chi0(i,j) - 0.01
               enddo
            enddo
         endif
         !
      ELSE
         !
         do i = 1, nat
           do j = 1, nat
              chi_aux(i,j)  = chi(i,j)
              chi0_aux(i,j) = chi0(i,j)
           enddo
         enddo
         CALL invmat_loc(chi_aux,chim1,nat)
         CALL invmat_loc(chi0_aux,chi0m1,nat)
         !
      ENDIF
      !
      !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!! Write the results  !!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      !
      ! OPEN file to write the results
      open(12,file='Hubbard_parameters.dat',status='unknown')
      !
      write(12,*)
      write(12,*) '   chi0 matrix :'
      do i = 1, nat
         write(12,'(8(x,f10.6))') (chi0(i,j),j=1,nat)
         write(12,*) 
      enddo
      !    
      write(12,*) '   chi matrix :'
      do i = 1, nat
         write(12,'(8(x,f10.6))') (chi(i,j),j=1,nat)
         write(12,*) 
      enddo
      !
      write(12,*)
      write(12,*) '   chi0^{-1} matrix :'
      do i = 1, nat
         write(12,'(8(x,f10.6))') (chi0m1(i,j),j=1,nat)
         write(12,*)
      enddo
      !
      write(12,*) '   chi^{-1} matrix :'
      do i = 1, nat
         write(12,'(8(x,f10.6))') (chim1(i,j),j=1,nat)
         write(12,*) 
      enddo
      write(12,*)
      !
      write(12,*) '   Hubbard matrix:'
      do i = 1, nat
         write(12,'(8(x,f10.6))') (chi0m1(i,j)-chim1(i,j),j=1,nat)
         write(12,*)
      enddo
      !
      nat0 = 0
      do i = 1,ntyp
         do j = 1,nat
            if (typ(j).eq.i) then
               write(12,*) ' type: ', i, ' U = ', chi0m1(j,j)-chim1(j,j)
               go to 15
            end if
         enddo
 15      continue
      enddo
      ! 
      close(12)
      !
      deallocate(chi, chi0, chim1, chi0m1, alpha, nn, dist, typ)
      deallocate(chi_aux, chi0_aux)
      deallocate(filename, filename0)
      deallocate(spin, spin_ref)
      deallocate(typ_ref)
      !
      STOP
      !
end program resp_mat

!----------------------------------------------------------------------
subroutine invmat_loc (a, a_inv, n)
!----------------------------------------------------------------------
   ! computes the inverse "a_inv" of matrix "a", both dimensioned (n,n)
   ! matrix "a" is unchanged on output - LAPACK
   !
   implicit none
   integer :: n, i, j
   real(kind=8) :: a(n, n), a_inv(n, n)
   !                                                                              
   integer, parameter :: lwork = 500000
   ! lwork is the dimension of work and ipiv. Must be >= n
   integer :: info, lda, ipiv (lwork)
   ! info=0: inversion was successful
   ! lda   : leading dimension (the same as n)
   ! ipiv  : work space for pivoting
   real(kind=8) :: work (lwork)
   !                                                                              
   if (n.gt.lwork) write(6,*) 'work space is too small ', n
   lda = n
   call DCOPY (n * n, a, 1, a_inv, 1)
   call DGETRF (n, n, a_inv, lda, ipiv, info)
   call DGETRI (n, a_inv, lda, ipiv, work, lwork, info)
   return
   !                                                                             
end subroutine invmat_loc
